//+--------------------------------------------------------------------------+
//|                                                                          |
//|                                GDImage.h                                 |
//|                                                                          |
//|         C++ GDImage API encapsulation constants and declarations.        |
//|                                                                          |
//|                               Version 5.16                               |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                                                                          |
//|                         Author Patrice TERRIER                           |
//|            8 Domaine de Rochagnon. 38800 Champagnier  FRANCE             |
//|                       http://www.zapsolution.com                         |
//|                    E-mail: pterrier@zapsolution.com                      |
//|                                                                          |
//|                   copyright (c) 2010 Patrice TERRIER                     |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|          C++ translation started on : 08-03-2010                         |
//|                        Last revised : 08-08-2010                         |
//+--------------------------------------------------------------------------+

#include <windows.h>
//
#include <iostream>
#include <string.h>

// Macro definitions.
#define int_proc typedef int (__stdcall *zProc)
#define string_proc typedef char* (__stdcall *zProc)
#define void_proc typedef void (__stdcall *zProc)
#define handle_proc typedef HANDLE (__stdcall *zProc)

//
// Constants.
#define zLim              '|'
#define Anti              '\\'
#define Dot               '.'
#define GDImageClassName  "ZIMAGECTRL"  // Class name for GDImage standard control.
#define GLImageClassName  "ZOPENGLCTRL" // Class name for OpenGL control.

// Anchor constants
#define ANCHOR_NONE                0
#define ANCHOR_LEFT                ANCHOR_NONE
#define ANCHOR_WIDTH               1
#define ANCHOR_RIGHT               2
#define ANCHOR_CENTER_HORZ         3
#define ANCHOR_HEIGHT              4
#define ANCHOR_HEIGHT_WIDTH        5
#define ANCHOR_HEIGHT_RIGHT        6
#define ANCHOR_BOTTOM              7
#define ANCHOR_BOTTOM_WIDTH        8
#define ANCHOR_BOTTOM_RIGHT        9
#define ANCHOR_CENTER_HORZ_BOTTOM  10
#define ANCHOR_CENTER_VERT         11
#define ANCHOR_CENTER_VERT_RIGHT   12
#define ANCHOR_CENTER              13

#define ZM_Size            WM USER + 1001
#define ZM_Crop            WM USER + 1002
#define ZM_CENTER_DIALOG   WM USER + 1003
#define ZM_CropRedEye      WM USER + 1004

// Constant prefixed with ZD_ apply to DRAWING functions
#define ZD_TRANSCOLOR      0xFF00FF     // Magenta color that is the default GDImage transparent color
#define ZD_TOPLEFTCOLOR    -1           // Pixel color at location 0,0
//
// ZD DrawText constants
#define ZD_TextHorzUp      0            // Horizontal text orientation
#define ZD_TextVertUp      4            // Vertical text orientation from bottom to top
#define ZD_TextVertDn      8            // Vertical text orientation from top to bottom
#define ZD_HIDE            0            // FALSE
#define ZD_SHOW            1            // TRUE

// GDImage overlay Generic STYLE must be a 2^
#define ZS_HIDDEN          ZD_HIDE
#define ZS_VISIBLE         ZD_SHOW      // VISIBLE, same as %WS VISIBLE
#define ZS_SCROLL          2            // MOVE with scroll bars, default is FIX (do not scroll)
#define ZS_DRAFT           4            // Draft drawing mode

// GDImage drawing style for all the ZD_Drawxxx functions
#define ZD_DRAW_OUTLINE    1
#define ZD_DRAW_FILLED     2
#define ZD_DRAW_3DIN       3
#define ZD_DRAW_3DOUT      4
#define ZD_DRAW_OPEN       5

#define ZD_DRAW_REDRAW     -1           // Forces immediate redraw of parent object
#define ZD_DRAW_DEFERRED   0            // Defers redraw of parent object

// Sprite objects:
#define OBJECT_TEXT        1
#define OBJECT_ELLIPSE     2
#define OBJECT_RECT        3
#define OBJECT_CURVE       4
#define OBJECT_ARROW       5
#define OBJECT_POLYLINE    6
#define OBJECT_BITMAP      7
#define OBJECT_BEZIER      8
#define OBJECT_POLYPOLYGON 9
#define OBJECT_TEXTBITMAP  10

// Internal GDImage structure being used to store details about each sprite object.
#define      METADATA_SIZE    2048      //See it in GDImage.inc
typedef struct {
     HWND    hWnd;        
     LONG    objType;     
     LONG    metacount;   
     char    metadata[METADATA_SIZE];

     char    useFont[64];
     char    extrabyte[64];

     LONG    useSize;     
     LONG    fontFam;     
     LONG    curFont;     
     LONG    strFormat;   
     LONG    useARGB;     
     LONG    use3D;       
     LONG    x1;          // x top coordinate
     LONG    y1;          // y top coordinate
     LONG    x2;          // object width
     LONG    y2;          // object height
     LONG    visible;     
     LONG    order;       
     BYTE    locked;      

     LONG    ID;              
     LONG    Style;       
     LONG    Bitmap;      
     LONG    Region;      
     char    objLabel[64];
     short   xCapture;
     short   yCapture;
     BYTE    opacity;     // PNG variable opacity
     float   scale;         
     LONG    clone;         
     short   xP;            
     short   yP;            
     WORD    angle;      
     BYTE    quality;          
     BYTE    flipmode;         
     BYTE    userotate;        
     BYTE    selected;         
     BYTE    hidden;           
          
     BYTE    brightness;       
     BYTE    contrast;         
     BYTE    red;              
     BYTE    green;            
     BYTE    blue;             
     BYTE    hue;              
          
     BYTE    FrameCount;       
     BYTE    FrameToUse;       
          
     BYTE    anchor;           
     SHORT   nLeft;   
     SHORT   nTop;       
     SHORT   nRight;     
     SHORT   nBottom;    
     SHORT   centerx;    
     SHORT   centery;    

     LONG    linked;      

     char    reserved[2];
} ZOBJECT; // 2368 bytes
// WARNING in case of change into the ZOBJECT structure that would impact its size,
// don't forget to update the "zItemFromID" and "zCloneFromID" functions!!!

HMODULE g_GDImage_DLL;

// Load the GDImage DLL.
int LoadDLL_GDImage() {
    static int nRet;
    if (nRet == 0) {
       g_GDImage_DLL = LoadLibraryA ("GDImage");
       if (g_GDImage_DLL != 0) { nRet = -1; }
    }
    return nRet;
}

// Unlock the DLL
int ZI_LoadDLL (
    std::string sUserKey        // The private user key.
    ) {

    int nRet = 0;

    if (g_GDImage_DLL == 0) { LoadDLL_GDImage(); }

    if (g_GDImage_DLL) {
        int_proc (char*);
        zProc hPROC = (zProc) GetProcAddress(g_GDImage_DLL, "ZI_LoadDLL");
        if (hPROC) {
            nRet = hPROC((char*) sUserKey.c_str());
        }
        else {
            FreeLibrary(g_GDImage_DLL);
        }
    }
    return nRet; // 0 = means failure, any other value means success.
}

// Register the GDImage window classes.
// Must be done in the WinMain section BEFORE USING any of the GDImage function.
// Parameter: None
int RegisterGDImageClass(
    ) {

    int nRet = 0;

    if (g_GDImage_DLL == 0) { LoadDLL_GDImage(); }

    if (g_GDImage_DLL) {
        int_proc ();
        zProc hPROC = (zProc) GetProcAddress(g_GDImage_DLL, "RegisterGDImageClass");
        if (hPROC) {
            nRet = hPROC();
        }
        else {
            FreeLibrary(g_GDImage_DLL);
        }
    }
    return nRet; // 0 = means failure, any other value means registered.
}

// Get the GDImage.dll internal version number
string ZI_Version (
    ) {

    std::string sText;

    if (g_GDImage_DLL) {
        string_proc ();
        zProc hPROC = (zProc) GetProcAddress(g_GDImage_DLL, "ZI_Version");
        if (hPROC) {
            sText = hPROC();
        }
    }
    return sText; // Version number of the form "#.#"
}

// Enable/Disable drawing of the WinLIFT skin background in the specified GDImage graphic control. 
void ZI_UseWinLIFTbackground (
    HWND hGDImageCtrl,          // The GDimage control handle.
    int TrueOrFalse,            // The enable/disable boolean value to use.
    int RedrawParent            // TRUE causes immediate redraw of the object parent.
    ) {

    if (g_GDImage_DLL) {
        void_proc (HWND, int, int);
        zProc hPROC = (zProc) GetProcAddress(g_GDImage_DLL, "ZI_UseWinLIFTbackground");
        if (hPROC) {
            hPROC(hGDImageCtrl, TrueOrFalse, RedrawParent);
        }
    }
}

// To force immediate refresh of a GDImage control display
void ZI_UpdateWindow (  
    HWND hGDImageCtrl,          // The GDimage control handle.
    int EraseFlag               // Boolean flag to erase first the background.
    ) {

    if (g_GDImage_DLL) {
        void_proc (HWND, int);
        zProc hPROC = (zProc) GetProcAddress(g_GDImage_DLL, "ZI_UpdateWindow");
        if (hPROC) {
            hPROC(hGDImageCtrl, EraseFlag);
        }
    }
}

// Apply a scale factor to resize a sprite on the fly
// Note: the real size of the sprite bitmap stay unchanged, it just affect the way it is being shown.
void ZD_SetObjectScale (  
    int ObjID,                  // The unique sprite object identifier.
    float rScaleValue           // The scale factor 0.5, 0.75, 0.80, etc.
    ) {

    if (g_GDImage_DLL) {
        void_proc (int, float);
        zProc hPROC = (zProc) GetProcAddress(g_GDImage_DLL, "ZD_SetObjectScale");
        if (hPROC) {
            hPROC(ObjID, rScaleValue);
        }
    }
}

// Disable (cancel) the drawing of a rectangular section.
// See also: ZI EnableCropRectangle.
void ZI_DisableCropRectangle (  
    HWND hGDImageCtrl           // The GDimage control handle.
    ) {

    if (g_GDImage_DLL) {
        void_proc (HWND);
        zProc hPROC = (zProc) GetProcAddress(g_GDImage_DLL, "ZI_DisableCropRectangle");
        if (hPROC) {
            hPROC(hGDImageCtrl);
        }
    }
}

// The Crop tool status mode.
// Return: True = Crop status is On, False = Crop status is false.
int ZI_GetToolCropStatus (  
    HWND hGDImageCtrl           // The GDimage control handle.
    ) {

    int nRet = 0;

    if (g_GDImage_DLL) {
        int_proc (HWND);
        zProc hPROC = (zProc) GetProcAddress(g_GDImage_DLL, "ZI_GetToolCropStatus");
        if (hPROC) {
            nRet = hPROC(hGDImageCtrl);
        }
    }
    return nRet; // 0 = means failure, any other value means success.
}

// Retrieve the immediate parent of the overlay object
// Return: the GDImage control handle of the parent.
HWND ZD_GetObjectParent (  
    int ObjID                   // The unique object IDentifier (each object MUST USE A UNIQUE one).
    ) {

    HWND hCtrl = 0;

    if (g_GDImage_DLL) {
        hwnd_proc (int);
        zProc hPROC = (zProc) GetProcAddress(g_GDImage_DLL, "ZD_GetObjectParent");
        if (hPROC) {
            hCtrl = hPROC(ObjID);
        }
    }
    return hCtrl;
}

//4.05
// Useful to embend an image into a fixed size square to perform image rotation from the center.
// Example 512 x 512 or 1024 x 1024 or 2048 x 2048 or 1024 x 512 (for texture creation).
HANDLE ZI_FitBitmapFromFile ( 
    std::string sFullPath,         // Full path name to image.
    int Xin,                       // Width of the container.
    int Yin                        // Height of the container.
    ) {

    HANDLE hBitmap = 0;

    if (g_GDImage_DLL) {
        handle_proc (char*, int, int);
        zProc hPROC = (zProc) GetProcAddress(g_GDImage_DLL, "ZI_FitBitmapFromFile");
        if (hPROC) {
            hBitmap = hPROC((char*) sFullPath.c_str(), Xin, Yin);
        }
    }
    return hBitmap;
}

// Computes location and size to stretch a bitmap preserving its aspect.
void ZI_Iconise (
    int xPicSize,               // The real image width.
    int yPicSize,               // The real image height.
    int xCell,                  // The fit into wdith.
    int yCell,                  // The fit inot height.
    int &xPos,                  // New X computed value.
    int &yPos,                  // New Y computed value.
    int &xSiz,                  // New Width computed value.
    int &ySiz                   // New Height computed value.
    ) {

    xPos = 0, yPos = 0; xSiz = 0, ySiz = 0;
    if (g_GDImage_DLL) {
        void_proc (int, int, int, int, int*, int*, int*, int*);
        zProc hPROC = (zProc) GetProcAddress(g_GDImage_DLL, "ZI_Iconise");
        if (hPROC) {
            hPROC(xPicSize, yPicSize, xCell, yCell, &xPos, &yPos, &xSiz, &ySiz); // Passage by reference.
        }
    }
}

// Create a sprite BITMAP (overlay object).
// Return: TRUE (-1) if success, FALSE (0) if error.
int ZD_DrawBitmapToCtrl (  
    HWND hGDImageCtrl,          // The GDImage control handle.
    int x,                      // The X coordinate.
    int y,                      // The Y coordinate.
    HANDLE hBitmap,             // The bitmap handle.
    int ColrARGB,               // ARGB color with Alpha channel (0-255).
    int ObjID,                  // Unique onlect IDentifier.
    int ZS_Style  
    ) {

    int nRet = 0;

    if (g_GDImage_DLL) {
        int_proc (HWND, int, int, HANDLE, int, int, int);
        zProc hPROC = (zProc) GetProcAddress(g_GDImage_DLL, "ZD_DrawBitmapToCtrl");
        if (hPROC) {
            nRet = hPROC(hGDImageCtrl, x, y, hBitmap, ColrARGB, ObjID, ZS_Style); 
        }
    }
    return nRet;
}

// Retrieve bitmap size from a supported GDI+ disk file (do not load the image)
void ZI_GetImageSizeFromFile (  
    string sFullPath,           // The name of the graphic file to detect size
    int &imgWidth,              // Width of the picture
    int &imgHeight              // Height of the picture
    ) {

    imgWidth = 0, imgHeight = 0;
    if (g_GDImage_DLL) {
        void_proc (char*, int*, int*);
        zProc hPROC = (zProc) GetProcAddress(g_GDImage_DLL, "ZI_GetImageSizeFromFile");
        if (hPROC) {
            hPROC((char*) sFullPath.c_str(), &imgWidth, &imgHeight); // Passage by reference.
        }
    }
}

// To GET the Width and Height of a STANDARD BITMAP
void ZI_GetBitmapSize (  
    HANDLE hBitmap,             // Handle to a standard bitmap.
    int &BitmapWidth,           // The width of the bitmap.
    int &BitmapHeight           // The Height of the bitmap.
    ) {

    BitmapWidth = 0, BitmapHeight = 0;
    if (g_GDImage_DLL) {
        void_proc (HANDLE, int*, int*);
        zProc hPROC = (zProc) GetProcAddress(g_GDImage_DLL, "ZI_GetBitmapSize");
        if (hPROC) {
            hPROC(hBitmap, &BitmapWidth, &BitmapHeight); // Passage by reference.
        }
    }
}

// Compute an ARGB color to ensure compatibility with GDI+
// Return: 32-bit ARGB color.
int ZD_ColorARGB (  
    BYTE AlphaChannel,          // The translucency level ranging from 0 to 255.
    DWORD ColrRGB               // The classical RGB() color to use.
    ) {

    int nRet = 0;

    if (g_GDImage_DLL) {
        int_proc (BYTE, DWORD);
        zProc hPROC = (zProc) GetProcAddress(g_GDImage_DLL, "ZD_ColorARGB");
        if (hPROC) {
            nRet = hPROC(AlphaChannel, ColrRGB);
        }
    }
    return nRet;
}

// Retrieve the rotation angle of a GDImage sprite object.
// Return: the rotation angle in degree (0-360).
int ZD_GetObjectAngle (  
    int ObjID  
    ) {

    int nRet = 0;

    if (g_GDImage_DLL) {
        int_proc (int);
        zProc hPROC = (zProc) GetProcAddress(g_GDImage_DLL, "ZD_GetObjectAngle");
        if (hPROC) {
            nRet = hPROC(ObjID);
        }
    }
    return nRet;
}

// Set the rotation angle of a GDImage sprite object.
// Remark: full rotation angle applies only to bitmap objects (OBJECT_BITMAP and OBJECT_TEXTBITMAP).
void ZD_SetObjectAngle (  
  int ObjID,         // The unique sprite object identifier.
  int AngleDegree,   // Value must fit within the 0-360 degree range.
  int RedrawParent   // TRUE causes immediate redraw of the object parent.
  ) {

    if (g_GDImage_DLL) {
        void_proc (int, int, int);
        zProc hPROC = (zProc) GetProcAddress(g_GDImage_DLL, "ZD_SetObjectAngle");
        if (hPROC) {
            hPROC(ObjID, AngleDegree, RedrawParent);
        }
    }
}

// To anchor a specific GDImage sprite object.
// Note: it uses a limited subset of the ANCHOR_flags,
//       because you can't change the width or the height of a sprite object with any of ANCHOR_WIDTH or ANCHOR_HEIGHT
int ZD_SetObjectAnchorMode (  
    int nObjID,  
    int AnchorMode  
    ) {

    int nRet = 0;

    if (g_GDImage_DLL) {
        int_proc (int, int);
        zProc hPROC = (zProc) GetProcAddress(g_GDImage_DLL, "ZD_SetObjectAnchorMode");
        if (hPROC) {
            nRet = hPROC(nObjID, AnchorMode);
        }
    }
    return nRet;
}

// Put the focus on a specific GDImage sprite object.
// Remark: dito a left mouse button clic on a GDImage sprite object.
void ZI_SetObjectFocusID (  
    int nObjID                  // The unique object IDentifier.
    ) {

    if (g_GDImage_DLL) {
        void_proc (int);
        zProc hPROC = (zProc) GetProcAddress(g_GDImage_DLL, "ZI_SetObjectFocusID");
        if (hPROC) {
            hPROC(nObjID);
        }
    }
}

// Show or Hide a specific GDImage overlay object.
void ZD_SetObjectVisibility (  
    int nObjID,                 // The unique object IDentifier (each object MUST USE A UNIQUE one).
    int ShowOrHide              // A boolean TRUE/FALSE flag to show or hide the object.
    ) {

    if (g_GDImage_DLL) {
        void_proc (int, int);
        zProc hPROC = (zProc) GetProcAddress(g_GDImage_DLL, "ZD_SetObjectVisibility");
        if (hPROC) {
            hPROC(nObjID, ShowOrHide);
        }
    }
}

// Display the common file selector dialog using thumbnail preview for the supported graphic format.
// Return: the FullPathName of the selected picture
string ZI_LoadDialog (
    HWND hParent                // The parent window handle.
    ) {

    std::string sFileName;

    if (g_GDImage_DLL) {
        string_proc (HWND);
        zProc hPROC = (zProc) GetProcAddress(g_GDImage_DLL, "ZI_LoadDialog");
        if (hPROC) {
            sFileName = hPROC(hParent);
        }
    }
    return sFileName;
}

// To retrieve the image size of specific GDimage control
void ZI_GetImageSizeFromControl (  
    HWND hGDImageCtrl,          // The GDimage control handle.
    int &imgW,                  // Width of the picture.
    int &imgH                   // Height of the picture.
    ) {

    imgW = 0, imgH = 0;
    if (g_GDImage_DLL) {
        void_proc (HWND, int*, int*);
        zProc hPROC = (zProc) GetProcAddress(g_GDImage_DLL, "ZI_GetImageSizeFromControl");
        if (hPROC) {
            hPROC(hGDImageCtrl, &imgW, &imgH);
        }
    }
}

// Create a valid GDI BITMAP handle from FILE using any of the supported GDImage format.
// Return: a valid GDI bitmap handle (do not confuse GDI bitmap handle and GDI+ Image handle).
// Important: you must use DeleteObject(GDIBitmapHandle) when you do not need it anymore.
HANDLE ZI_CreateBitmapFromFile (  
    string sFullPath,           // The name of an image file.
    int &bmW,                   // The width of the bitmap.
    int &bmH                    // The Height of the bitmap.
    ) {

    HANDLE hBitmap = 0;
    bmW = 0, bmH = 0;

    if (g_GDImage_DLL) {
        handle_proc (char*, int*, int*);
        zProc hPROC = (zProc) GetProcAddress(g_GDImage_DLL, "ZI_CreateBitmapFromFile");
        if (hPROC) {
            hBitmap = hPROC((char*) sFullPath.c_str(), &bmW, &bmH);
        }
    }
    return hBitmap;
}        

// Set the mouse lock status of a specific sprite object.
void ZD_SetObjectLocked (  
    int ObjID,                  // The unique sprite object identifier.
    int TrueFalse               // The TRUE or FALSE lock mode.
    ) {

    if (g_GDImage_DLL) {
        void_proc (int, int);
        zProc hPROC = (zProc) GetProcAddress(g_GDImage_DLL, "ZD_SetObjectLocked");
        if (hPROC) {
            hPROC(ObjID, TrueFalse);
        }
    }
}
